/*
 * @(#)AccessBridgePackages.h	1.37 04/06/29
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* 
 * Header file for packages of paramaters passed between Java Accessibility
 * and native Assistive Technologies
 */

#ifndef __AccessBridgePackages_H__
#define __AccessBridgePackages_H__

#include <jni.h>
//#include <javaString.h>
#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif

    // versioning
    //
#define ACCESS_BRIDGE_VERSION  L"1.2 GA"
#define JAVA_ACCESS_BRIDGE_VERSION      ACCESS_BRIDGE_VERSION
#define WINDOW_ACCESS_BRIDGE_VERSION    ACCESS_BRIDGE_VERSION


#define MAX_STRING_SIZE   1024
#define SHORT_STRING_SIZE   256

    // object types
    typedef jobject AccessibleContext;
    typedef jobject AccessibleText;
    typedef jobject AccessibleValue;
    typedef jobject AccessibleSelection;
    typedef jobject Java_Object;
    typedef jobject PropertyChangeEvent;
    typedef jobject FocusEvent;
    typedef jobject CaretEvent;
    typedef jobject MouseEvent;
    typedef jobject MenuEvent;
    typedef jobject AccessibleTable;
    typedef jobject AccessibleHyperlink;
    typedef jobject AccessibleHypertext;

    /**
******************************************************
*  Java event types
******************************************************
*/

#define cPropertyChangeEvent (jlong) 1						// 1

#define cFocusGainedEvent (jlong) 2							// 2
#define cFocusLostEvent (jlong) 4							// 4

#define cCaretUpdateEvent (jlong) 8							// 8

#define cMouseClickedEvent (jlong) 16						// 10
#define cMouseEnteredEvent (jlong) 32						// 20
#define cMouseExitedEvent (jlong) 64						// 40
#define cMousePressedEvent (jlong) 128						// 80
#define cMouseReleasedEvent (jlong) 256						// 100

#define cMenuCanceledEvent (jlong) 512						// 200
#define cMenuDeselectedEvent (jlong) 1024					// 400
#define cMenuSelectedEvent (jlong) 2048						// 800
#define cPopupMenuCanceledEvent (jlong) 4096				// 1000
#define cPopupMenuWillBecomeInvisibleEvent (jlong) 8192		// 2000
#define cPopupMenuWillBecomeVisibleEvent (jlong) 16384		// 4000

    /**
******************************************************
*  Accessible Roles
*      Defines all AccessibleRoles in Local.US
******************************************************
*/

    /**
     * Object is used to alert the user about something.
     */
#define ACCESSIBLE_ALERT L"alert"   

    /**
     * The header for a column of data.
     */
#define ACCESSIBLE_COLUMN_HEADER L"column header"

    /** 
     * Object that can be drawn into and is used to trap
     * events.
     * see ACCESSIBLE_FRAME
     * see ACCESSIBLE_GLASS_PANE
     * see ACCESSIBLE_LAYERED_PANE
     */     
#define ACCESSIBLE_CANVAS L"canvas"

    /** 
     * A list of choices the user can select from.  Also optionally 
     * allows the user to enter a choice of their own.
     */     
#define ACCESSIBLE_COMBO_BOX L"combo box"

    /** 
     * An iconified internal frame in a DESKTOP_PANE.
     * see ACCESSIBLE_DESKTOP_PANE
     * see ACCESSIBLE_INTERNAL_FRAME
     */
#define ACCESSIBLE_DESKTOP_ICON L"desktop icon"

    /** 
     * A frame-like object that is clipped by a desktop pane.  The
     * desktop pane, internal frame, and desktop icon objects are 
     * often used to create multiple document interfaces within an
     * application.
     * see ACCESSIBLE_DESKTOP_ICON
     * see ACCESSIBLE_DESKTOP_PANE
     * see ACCESSIBLE_FRAME
     */
#define ACCESSIBLE_INTERNAL_FRAME L"internal frame"

    /**
     * A pane that supports internal frames and 
     * iconified versions of those internal frames.
     * see ACCESSIBLE_DESKTOP_ICON
     * see ACCESSIBLE_INTERNAL_FRAME
     */
#define ACCESSIBLE_DESKTOP_PANE L"desktop pane"

    /** 
     * A specialized pane whose primary use is inside a DIALOG
     * see ACCESSIBLE_DIALOG
     */
#define ACCESSIBLE_OPTION_PANE L"option pane"

    /** 
     * A top level window with no title or border.
     * see ACCESSIBLE_FRAME
     * see ACCESSIBLE_DIALOG
     */
#define ACCESSIBLE_WINDOW L"window"

    /**
     * A top level window with a title bar, border, menu bar, etc.  It is
     * often used as the primary window for an application.
     * see ACCESSIBLE_DIALOG
     * see ACCESSIBLE_CANVAS
     * see ACCESSIBLE_WINDOW
     */
#define ACCESSIBLE_FRAME L"frame"

    /** 
     * A top level window with title bar and a border.  A dialog is similar 
     * to a frame, but it has fewer properties and is often used as a 
     * secondary window for an application.
     * see ACCESSIBLE_FRAME
     * see ACCESSIBLE_WINDOW
     */
#define ACCESSIBLE_DIALOG L"dialog"

    /**
     * A specialized dialog that lets the user choose a color.
     */
#define ACCESSIBLE_COLOR_CHOOSER L"color chooser"


    /**
     * A pane that allows the user to navigate through 
     * and select the contents of a directory.  May be used
     * by a file chooser.
     * see ACCESSIBLE_FILE_CHOOSER
     */
#define ACCESSIBLE_DIRECTORY_PANE L"directory pane"

    /**
     * A specialized dialog that displays the files in the directory
     * and lets the user select a file, browse a different directory,
     * or specify a filename.  May use the directory pane to show the
     * contents of a directory.
     * see ACCESSIBLE_DIRECTORY_PANE
     */
#define ACCESSIBLE_FILE_CHOOSER L"file chooser"

    /** 
     * An object that fills up space in a user interface.  It is often
     * used in interfaces to tweak the spacing between components,
     * but serves no other purpose.
     */
#define ACCESSIBLE_FILLER L"filler"

    /**
     * A hypertext anchor
     */
#define ACCESSIBLE_HYPERLINK L"hyperlink"

    /**
     * A small fixed size picture, typically used to decorate components.
     */
#define ACCESSIBLE_ICON L"icon"

    /** 
     * An object used to present an icon or short string in an interface.
     */
#define ACCESSIBLE_LABEL L"label"

    /**
     * A specialized pane that has a glass pane and a layered pane as its
     * children.
     * see ACCESSIBLE_GLASS_PANE
     * see ACCESSIBLE_LAYERED_PANE
     */
#define ACCESSIBLE_ROOT_PANE L"root pane"

    /**
     * A pane that is guaranteed to be painted on top
     * of all panes beneath it.
     * see ACCESSIBLE_ROOT_PANE
     * see ACCESSIBLE_CANVAS
     */
#define ACCESSIBLE_GLASS_PANE L"glass pane"

    /** 
     * A specialized pane that allows its children to be drawn in layers,
     * providing a form of stacking order.  This is usually the pane that
     * holds the menu bar as well as the pane that contains most of the
     * visual components in a window.
     * see ACCESSIBLE_GLASS_PANE
     * see ACCESSIBLE_ROOT_PANE
     */
#define ACCESSIBLE_LAYERED_PANE L"layered pane"

    /**
     * An object that presents a list of objects to the user and allows the
     * user to select one or more of them.  A list is usually contained
     * within a scroll pane.
     * see ACCESSIBLE_SCROLL_PANE
     * see ACCESSIBLE_LIST_ITEM
     */
#define ACCESSIBLE_LIST L"list"

    /**
     * An object that presents an element in a list.  A list is usually
     * contained within a scroll pane.
     * see ACCESSIBLE_SCROLL_PANE
     * see ACCESSIBLE_LIST
     */
#define ACCESSIBLE_LIST_ITEM L"list item"

    /**
     * An object usually drawn at the top of the primary dialog box of
     * an application that contains a list of menus the user can choose
     * from.  For example, a menu bar might contain menus for "File,"
     * "Edit," and "Help."
     * see ACCESSIBLE_MENU
     * see ACCESSIBLE_POPUP_MENU
     * see ACCESSIBLE_LAYERED_PANE
     */
#define ACCESSIBLE_MENU_BAR L"menu bar"

    /** 
     * A temporary window that is usually used to offer the user a 
     * list of choices, and then hides when the user selects one of
     * those choices.
     * see ACCESSIBLE_MENU
     * see ACCESSIBLE_MENU_ITEM
     */
#define ACCESSIBLE_POPUP_MENU L"popup menu"

    /** 
     * An object usually found inside a menu bar that contains a list
     * of actions the user can choose from.  A menu can have any object
     * as its children, but most often they are menu items, other menus,
     * or rudimentary objects such as radio buttons, check boxes, or
     * separators.  For example, an application may have an "Edit" menu 
     * that contains menu items for "Cut" and "Paste."
     * see ACCESSIBLE_MENU_BAR
     * see ACCESSIBLE_MENU_ITEM
     * see ACCESSIBLE_SEPARATOR
     * see ACCESSIBLE_RADIO_BUTTON
     * see ACCESSIBLE_CHECK_BOX
     * see ACCESSIBLE_POPUP_MENU
     */    
#define ACCESSIBLE_MENU L"menu"

    /**
     * An object usually contained in a menu that presents an action 
     * the user can choose.  For example, the "Cut" menu item in an
     * "Edit" menu would be an action the user can select to cut the
     * selected area of text in a document.
     * see ACCESSIBLE_MENU_BAR
     * see ACCESSIBLE_SEPARATOR
     * see ACCESSIBLE_POPUP_MENU
     */
#define ACCESSIBLE_MENU_ITEM L"menu item"

    /**
     * An object usually contained in a menu to provide a visual
     * and logical separation of the contents in a menu.  For example,
     * the "File" menu of an application might contain menu items for
     * "Open," "Close," and "Exit," and will place a separator between
     * "Close" and "Exit" menu items.
     * see ACCESSIBLE_MENU
     * see ACCESSIBLE_MENU_ITEM
     */
#define ACCESSIBLE_SEPARATOR L"separator"

    /**
     * An object that presents a series of panels (or page tabs), one at a 
     * time, through some mechanism provided by the object.  The most common 
     * mechanism is a list of tabs at the top of the panel.  The children of
     * a page tab list are all page tabs.
     * see ACCESSIBLE_PAGE_TAB
     */
#define ACCESSIBLE_PAGE_TAB_LIST L"page tab list"

    /**
     * An object that is a child of a page tab list.  Its sole child is
     * the panel that is to be presented to the user when the user 
     * selects the page tab from the list of tabs in the page tab list.
     * see ACCESSIBLE_PAGE_TAB_LIST
     */
#define ACCESSIBLE_PAGE_TAB L"page tab"

    /**
     * A generic container that is often used to group objects.
     */
#define ACCESSIBLE_PANEL L"panel"

    /**
     * An object used to indicate how much of a task has been completed.
     */
#define ACCESSIBLE_PROGRESS_BAR L"progress bar"

    /**
     * An object used to monitor the progress of an operation
     */
#define ACCESSIBLE_PROGRESS_MONITOR L"progress monitor"

    /**
     * A text object used for passwords, or other places where the 
     * text contents is not shown visibly to the user
     */
#define ACCESSIBLE_PASSWORD_TEXT L"password text"

    /**
     * An object the user can manipulate to tell the application to do
     * something.
     * see ACCESSIBLE_CHECK_BOX
     * see ACCESSIBLE_TOGGLE_BUTTON
     * see ACCESSIBLE_RADIO_BUTTON
     */
#define ACCESSIBLE_PUSH_BUTTON L"push button"
  
    /**
     * A specialized push button that can be checked or unchecked, but
     * does not provide a separate indicator for the current state.
     * see ACCESSIBLE_PUSH_BUTTON
     * see ACCESSIBLE_CHECK_BOX
     * see ACCESSIBLE_RADIO_BUTTON
     */
#define ACCESSIBLE_TOGGLE_BUTTON L"toggle button"

    /**
     * A choice that can be checked or unchecked and provides a 
     * separate indicator for the current state.
     * see ACCESSIBLE_PUSH_BUTTON
     * see ACCESSIBLE_TOGGLE_BUTTON
     * see ACCESSIBLE_RADIO_BUTTON
     */
#define ACCESSIBLE_CHECK_BOX L"check box"

    /**
     * A specialized check box that will cause other radio buttons in the
     * same group to become unchecked when this one is checked.  
     * see ACCESSIBLE_PUSH_BUTTON
     * see ACCESSIBLE_TOGGLE_BUTTON
     * see ACCESSIBLE_CHECK_BOX
     */
#define ACCESSIBLE_RADIO_BUTTON L"radio button"

    /**
     * The header for a row of data.
     */
#define ACCESSIBLE_ROW_HEADER L"row header"

    /**
     * An object that allows a user to incrementally view a large amount
     * of information.  Its children can include scroll bars and a viewport.
     * see ACCESSIBLE_SCROLL_BAR
     * see ACCESSIBLE_VIEWPORT
     */
#define ACCESSIBLE_SCROLL_PANE L"scroll pane"

    /** 
     * An object usually used to allow a user to incrementally view a
     * large amount of data.  Usually used only by a scroll pane.
     * see ACCESSIBLE_SCROLL_PANE
     */
#define ACCESSIBLE_SCROLL_BAR L"scroll bar"

    /**
     * An object usually used in a scroll pane.  It represents the portion 
     * of the entire data that the user can see.  As the user manipulates 
     * the scroll bars, the contents of the viewport can change.
     * see ACCESSIBLE_SCROLL_PANE
     */
#define ACCESSIBLE_VIEWPORT L"viewport"

    /**
     * An object that allows the user to select from a bounded range.  For
     * example, a slider might be used to select a number between 0 and 100.
     */    
#define ACCESSIBLE_SLIDER L"slider"

    /**
     * A specialized panel that presents two other panels at the same time.
     * Between the two panels is a divider the user can manipulate to make
     * one panel larger and the other panel smaller.
     */
#define ACCESSIBLE_SPLIT_PANE L"split pane"

    /**
     * An object used to present information in terms of rows and columns.
     * An example might include a spreadsheet application.
     */
#define ACCESSIBLE_TABLE L"table"

    /**
     * An object that presents text to the user.  The text is usually
     * editable by the user as opposed to a label.
     * see ACCESSIBLE_LABEL
     */
#define ACCESSIBLE_TEXT L"text"

    /**
     * An object used to present hierarchical information to the user.
     * The individual nodes in the tree can be collapsed and expanded
     * to provide selective disclosure of the tree's contents.
     */
#define ACCESSIBLE_TREE L"tree"

    /**
     * A bar or palette usually composed of push buttons or toggle buttons.
     * It is often used to provide the most frequently used functions for an
     * application.
     */
#define ACCESSIBLE_TOOL_BAR L"tool bar"

    /** 
     * An object that provides information about another object.  The 
     * accessibleDescription property of the tool tip is often displayed 
     * to the user in a small "help bubble" when the user causes the 
     * mouse to hover over the object associated with the tool tip.
     */
#define ACCESSIBLE_TOOL_TIP L"tool tip"

    /**
     * An AWT component, but nothing else is known about it.
     * see ACCESSIBLE_SWING_COMPONENT
     * see ACCESSIBLE_UNKNOWN
     */
#define ACCESSIBLE_AWT_COMPONENT L"awt component"

    /**
     * A Swing component, but nothing else is known about it.
     * see ACCESSIBLE_AWT_COMPONENT
     * see ACCESSIBLE_UNKNOWN
     */
#define ACCESSIBLE_SWING_COMPONENT L"swing component"

    /**
     * The object contains some Accessible information, but its role is
     * not known.
     * see ACCESSIBLE_AWT_COMPONENT
     * see ACCESSIBLE_SWING_COMPONENT
     */
#define ACCESSIBLE_UNKNOWN L"unknown"

    /**
     * A STATUS_BAR is an simple component that can contain
     * multiple labels of status information to the user.
     */
#define ACCESSIBLE_STATUS_BAR L"status bar"
    
    /**
     * A DATE_EDITOR is a component that allows users to edit
     * java.util.Date and java.util.Time objects
     */
#define ACCESSIBLE_DATE_EDITOR L"date editor"
    
    /**
     * A SPIN_BOX is a simple spinner component and its main use
     * is for simple numbers.
     */
#define ACCESSIBLE_SPIN_BOX L"spin box"
    
    /**
     * A FONT_CHOOSER is a component that lets the user pick various
     * attributes for fonts.
     */
#define ACCESSIBLE_FONT_CHOOSER L"font chooser"
    
    /**
     * A GROUP_BOX is a simple container that contains a border
     * around it and contains components inside it.
     */
#define ACCESSIBLE_GROUP_BOX L"group box"

    /**
     * A text header
     */
#define ACCESSIBLE_HEADER = L"header"
 
    /**
     * A text footer
     */
#define ACCESSIBLE_FOOTER = L"footer"
 
    /**
     * A text paragraph
     */
#define ACCESSIBLE_PARAGRAPH = L"paragraph"
 
    /**
     * A ruler is an object used to measure distance
     */
#define ACCESSIBLE_RULER = L"puler"
 
    /**
     * A role indicating the object acts as a formula for
     * calculating a value.  An example is a formula in
     * a spreadsheet cell.
     */
#define ACCESSIBLE_EDITBAR = L"editbar"


    /**
******************************************************
*  Accessibility event types
******************************************************
*/

#define cPropertyNameChangeEvent (jlong) 1						// 1
#define cPropertyDescriptionChangeEvent (jlong) 2				// 2
#define cPropertyStateChangeEvent (jlong) 4						// 4
#define cPropertyValueChangeEvent (jlong) 8						// 8
#define cPropertySelectionChangeEvent (jlong) 16				// 10
#define cPropertyTextChangeEvent (jlong) 32					   	// 20
#define cPropertyCaretChangeEvent (jlong) 64					// 40
#define cPropertyVisibleDataChangeEvent (jlong) 128				// 80
#define cPropertyChildChangeEvent (jlong) 256					// 100
#define cPropertyActiveDescendentChangeEvent (jlong) 512		// 200

#define cPropertyTableModelChangeEvent (jlong) 1024				// 400

    /**
******************************************************
*  optional AccessibleContext interfaces
*
* This version of the bridge reuses the accessibleValue
* field in the AccessibleContextInfo struct to represent
* additional optional interfaces that are supported by
* the Java AccessibleContext.  This is backwardly compatable
* because the old accessibleValue was set to the BOOL 
* value TRUE (i.e., 1) if the AccessibleValue interface is 
* supported.
******************************************************
*/

#define cAccessibleValueInterface (jlong) 1						// 1 << 1 (TRUE)
#define cAccessibleActionInterface (jlong) 2					// 1 << 2
#define cAccessibleComponentInterface (jlong) 4					// 1 << 3
#define cAccessibleSelectionInterface (jlong) 8					// 1 << 4
#define cAccessibleTableInterface (jlong) 16					// 1 << 5
#define cAccessibleTextInterface (jlong) 32						// 1 << 6
#define cAccessibleHypertextInterface (jlong) 64 				// 1 << 7


    /**
******************************************************
*  Accessibility information bundles
******************************************************
*/

    typedef struct AccessBridgeVersionInfoTag {
	wchar_t VMversion[SHORT_STRING_SIZE];               // output of "java -version"
	wchar_t bridgeJavaClassVersion[SHORT_STRING_SIZE];  // version of the AccessBridge.class
	wchar_t bridgeJavaDLLVersion[SHORT_STRING_SIZE];    // version of JavaAccessBridge.dll
	wchar_t bridgeWinDLLVersion[SHORT_STRING_SIZE];     // version of WindowsAccessBridge.dll
    } AccessBridgeVersionInfo;


    typedef struct AccessibleContextInfoTag {
	wchar_t name[MAX_STRING_SIZE];			// the AccessibleName of the object
	wchar_t description[MAX_STRING_SIZE];	// the AccessibleDescription of the object

	wchar_t role[SHORT_STRING_SIZE];		// localized AccesibleRole string
	wchar_t role_en_US[SHORT_STRING_SIZE];	// AccesibleRole string in the en_US locale
	wchar_t states[SHORT_STRING_SIZE];		// localized AccesibleStateSet string (comma separated)
	wchar_t states_en_US[SHORT_STRING_SIZE]; // AccesibleStateSet string in the en_US locale (comma separated)

	jint indexInParent;					// index of object in parent
	jint childrenCount;					// # of children, if any

	jint x;								// screen coords in pixels
	jint y;								// "
	jint width;							// pixel width of object
	jint height;						// pixel height of object
									
	BOOL accessibleComponent;			// flags for various additional
	BOOL accessibleAction;				//  Java Accessibility interfaces
	BOOL accessibleSelection;			//  FALSE if this object doesn't
	BOOL accessibleText;				//  implement the additional interface
        //  in question

	// BOOL accessibleValue;            // old BOOL indicating whether AccessibleValue is supported
	BOOL accessibleInterfaces;			// new bitfield containing additional interface flags

    } AccessibleContextInfo;



    // AccessibleText packages
    typedef struct AccessibleTextInfoTag {
	jint charCount;						// # of characters in this text object
	jint caretIndex;					// index of caret
	jint indexAtPoint;					// index at the passsed in point
    } AccessibleTextInfo;

    typedef struct AccessibleTextItemsInfoTag {
	wchar_t letter;
	wchar_t word[SHORT_STRING_SIZE];
	wchar_t sentence[MAX_STRING_SIZE];
    } AccessibleTextItemsInfo;

    typedef struct AccessibleTextSelectionInfoTag {
	jint selectionStartIndex;
	jint selectionEndIndex;
	wchar_t selectedText[MAX_STRING_SIZE];
    } AccessibleTextSelectionInfo;

    typedef struct AccessibleTextRectInfoTag {
	jint x;								// bounding rect of char at index
	jint y;								// "
	jint width;							// "
	jint height;						// "
    } AccessibleTextRectInfo;

    // standard attributes for text; note: tabstops are not supported
    typedef struct AccessibleTextAttributesInfoTag {
	BOOL bold;
        BOOL italic;
        BOOL underline;
        BOOL strikethrough;
        BOOL superscript;
        BOOL subscript;

        wchar_t backgroundColor[SHORT_STRING_SIZE];
        wchar_t foregroundColor[SHORT_STRING_SIZE];
        wchar_t fontFamily[SHORT_STRING_SIZE];
        jint fontSize;

        jint alignment;
        jint bidiLevel;

        jfloat firstLineIndent;
        jfloat leftIndent;
        jfloat rightIndent;
        jfloat lineSpacing;
        jfloat spaceAbove;
        jfloat spaceBelow;

        wchar_t fullAttributesString[MAX_STRING_SIZE];
    } AccessibleTextAttributesInfo;

    /**
******************************************************
*  IPC management typedefs
******************************************************
*/

#define cMemoryMappedNameSize	255

    /**
     * sent by the WindowsDLL -> the memory-mapped file is setup
     *
     */
    typedef struct MemoryMappedFileCreatedPackageTag {
	HWND bridgeWindow;		// redundant, but easier to get to here...
	char filename[cMemoryMappedNameSize];
    } MemoryMappedFileCreatedPackage;
    
    /**
     * sent when a new JavaVM attaches to the Bridge
     *
     */
    typedef struct JavaVMCreatedPackageTag {
	HWND bridgeWindow;
	long vmID;
    } JavaVMCreatedPackage;
    
    /**
     * sent when a JavaVM detatches from the Bridge
     *
     */
    typedef struct JavaVMDestroyedPackageTag {
	HWND bridgeWindow;
    } JavaVMDestroyedPackage;
    
    /**
     * sent when a new AT attaches to the Bridge
     *
     */
    typedef struct WindowsATCreatedPackageTag {
	HWND bridgeWindow;
    } WindowsATCreatedPackage;
    
    /**
     * sent when an AT detatches from the Bridge
     *
     */
    typedef struct WindowsATDestroyedPackageTag {
	HWND bridgeWindow;
    } WindowsATDestroyedPackage;
    
    
    /**
     * sent by JVM Bridges in response to a WindowsATCreate
     * message; saying "howdy, welcome to the neighborhood"
     *
     */
    typedef struct JavaVMPresentNotificationPackageTag {
	HWND bridgeWindow;
	long vmID;
    } JavaVMPresentNotificationPackage;
    
    /**
     * sent by AT Bridges in response to a JavaVMCreate
     * message; saying "howdy, welcome to the neighborhood"
     *
     */
    typedef struct WindowsATPresentNotificationPackageTag {
	HWND bridgeWindow;
    } WindowsATPresentNotificationPackage;
    
    
    /**
******************************************************
*  Core packages
******************************************************
*/

    typedef struct ReleaseJavaObjectPackageTag {
	long vmID;
	jobject object;
    } ReleaseJavaObjectPackage; 

    typedef struct GetAccessBridgeVersionPackageTag {
	long vmID;                    // can't get VM info w/out a VM!
        AccessBridgeVersionInfo rVersionInfo;
    } GetAccessBridgeVersionPackage;

    typedef struct IsSameObjectPackageTag {
	long vmID;
	jobject obj1;
	jobject obj2;
	jboolean rResult;
    } IsSameObjectPackage; 

    /**
******************************************************
*  Windows packages
******************************************************
*/

    typedef struct IsJavaWindowPackageTag {
	jint window;
	jboolean rResult;
    } IsJavaWindowPackage; 

    typedef struct GetAccessibleContextFromHWNDPackageTag {
	jint window;
	long rVMID;
	jobject rAccessibleContext;
    } GetAccessibleContextFromHWNDPackage; 

    typedef struct GetHWNDFromAccessibleContextPackageTag {
	jobject accessibleContext;
	HWND rHWND;
    } GetHWNDFromAccessibleContextPackage; 

    /**
******************************************************
*  AccessibleContext packages
******************************************************
*/

    typedef struct GetAccessibleContextAtPackageTag {
	jint x;
	jint y;
	long vmID;
	jobject AccessibleContext;		// look within this AC
	jobject rAccessibleContext;
    } GetAccessibleContextAtPackage; 

    typedef struct GetAccessibleContextWithFocusPackageTag {
	long rVMID;
	jobject rAccessibleContext;
    } GetAccessibleContextWithFocusPackage; 

    typedef struct GetAccessibleContextInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	AccessibleContextInfo rAccessibleContextInfo;
    } GetAccessibleContextInfoPackage; 

    typedef struct GetAccessibleChildFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint childIndex;
	jobject rAccessibleContext;
    } GetAccessibleChildFromContextPackage; 

    typedef struct GetAccessibleParentFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jobject rAccessibleContext;
    } GetAccessibleParentFromContextPackage; 

    /**
******************************************************
*  AccessibleTable packages
******************************************************
*/

#define MAX_TABLE_SELECTIONS 64

    // table information
    typedef struct AccessibleTableInfoTag {
	jobject caption;	// AccesibleContext
        jobject summary;	// AccessibleContext
	jint rowCount;
	jint columnCount;
	jobject accessibleContext;
	jobject accessibleTable;
    } AccessibleTableInfo;

    typedef struct GetAccessibleTableInfoPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleTableInfo rTableInfo;
    } GetAccessibleTableInfoPackage;

    // table cell information
    typedef struct AccessibleTableCellInfoTag {
        jobject  accessibleContext;
        jint     index;
        jint     row;
        jint     column;
        jint     rowExtent;
        jint     columnExtent;
        jboolean isSelected;
    } AccessibleTableCellInfo;

    typedef struct GetAccessibleTableCellInfoPackageTag {
	long vmID;
	jobject accessibleTable;
	jint row;
	jint column;
	AccessibleTableCellInfo rTableCellInfo;
    } GetAccessibleTableCellInfoPackage; 

    typedef struct GetAccessibleTableRowHeaderPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleTableInfo rTableInfo;
    } GetAccessibleTableRowHeaderPackage;

    typedef struct GetAccessibleTableColumnHeaderPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleTableInfo rTableInfo;
    } GetAccessibleTableColumnHeaderPackage;

    typedef struct GetAccessibleTableRowDescriptionPackageTag {
	long vmID;
	jobject accessibleContext;
	jint row;
	jobject rAccessibleContext;
    } GetAccessibleTableRowDescriptionPackage;

    typedef struct GetAccessibleTableColumnDescriptionPackageTag {
	long vmID;
	jobject accessibleContext;
	jint column;
	jobject rAccessibleContext;
    } GetAccessibleTableColumnDescriptionPackage;

    typedef struct GetAccessibleTableRowSelectionCountPackageTag {
	long vmID;
	jobject accessibleTable;
	jint rCount;
    } GetAccessibleTableRowSelectionCountPackage;

    typedef struct IsAccessibleTableRowSelectedPackageTag {
	long vmID;
	jobject accessibleTable;
	jint row;
        jboolean rResult;
    } IsAccessibleTableRowSelectedPackage;

    typedef struct GetAccessibleTableRowSelectionsPackageTag {
	long vmID;
	jobject accessibleTable;
	jint count;
        jint rSelections[MAX_TABLE_SELECTIONS];
    } GetAccessibleTableRowSelectionsPackage;

    typedef struct GetAccessibleTableColumnSelectionCountPackageTag {
	long vmID;
	jobject accessibleTable;
	jint rCount;
    } GetAccessibleTableColumnSelectionCountPackage;

    typedef struct IsAccessibleTableColumnSelectedPackageTag {
	long vmID;
	jobject accessibleTable;
	jint column;
        jboolean rResult;
    } IsAccessibleTableColumnSelectedPackage;

    typedef struct GetAccessibleTableColumnSelectionsPackageTag {
	long vmID;
	jobject accessibleTable;
	jint count;
        jint rSelections[MAX_TABLE_SELECTIONS];
    } GetAccessibleTableColumnSelectionsPackage;


    typedef struct GetAccessibleTableRowPackageTag {
	long vmID;
	jobject accessibleTable;
	jint index;
        jint rRow;
    } GetAccessibleTableRowPackage;

    typedef struct GetAccessibleTableColumnPackageTag {
	long vmID;
	jobject accessibleTable;
	jint index;
        jint rColumn;
    } GetAccessibleTableColumnPackage;

    typedef struct GetAccessibleTableIndexPackageTag {
	long vmID;
	jobject accessibleTable;
	jint row;
        jint column;
	jint rIndex;
    } GetAccessibleTableIndexPackage;


    /**
******************************************************
*  AccessibleRelationSet packages
******************************************************
*/

#define MAX_RELATION_TARGETS 25
#define MAX_RELATIONS 5

    typedef struct AccessibleRelationInfoTag {
	wchar_t key[SHORT_STRING_SIZE];
	jint targetCount;
	jobject targets[MAX_RELATION_TARGETS];  // AccessibleContexts
    } AccessibleRelationInfo;

    typedef struct AccessibleRelationSetInfoTag {
	jint relationCount;
	AccessibleRelationInfo relations[MAX_RELATIONS];
    } AccessibleRelationSetInfo;

    typedef struct GetAccessibleRelationSetPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleRelationSetInfo rAccessibleRelationSetInfo;
    } GetAccessibleRelationSetPackage;

    /**
     ******************************************************
     *  AccessibleHypertext packagess
     ******************************************************
     */

#define MAX_HYPERLINKS		64	// maximum number of hyperlinks returned

    // hyperlink information
    typedef struct AccessibleHyperlinkInfoTag {
	wchar_t text[SHORT_STRING_SIZE]; // the hyperlink text
	jint startIndex;	//index in the hypertext document where the link begins
	jint endIndex;		//index in the hypertext document where the link ends
	jobject accessibleHyperlink; // AccessibleHyperlink object
    } AccessibleHyperlinkInfo;

    // hypertext information
    typedef struct AccessibleHypertextInfoTag {
	jint linkCount;		// number of hyperlinks
	AccessibleHyperlinkInfo links[MAX_HYPERLINKS];	// the hyperlinks
	jobject accessibleHypertext; // AccessibleHypertext object
    } AccessibleHypertextInfo;

    // struct for sending a message to get the hypertext for an AccessibleContext
    typedef struct GetAccessibleHypertextPackageTag {
	long vmID;              // the virtual machine ID
	jobject accessibleContext; // AccessibleContext with hypertext
	AccessibleHypertextInfo rAccessibleHypertextInfo; // returned hypertext
    } GetAccessibleHypertextPackage;

    // struct for sending an message to activate a hyperlink
    typedef struct ActivateAccessibleHyperlinkPackageTag {
	long vmID;              // the virtual machine ID
	jobject accessibleContext; // AccessibleContext containing the link
	jobject accessibleHyperlink; // the link to activate
        BOOL rResult;           // hyperlink activation return value
    } ActivateAccessibleHyperlinkPackage;
    
    // struct for sending a message to get the number of hyperlinks in a component
    typedef struct GetAccessibleHyperlinkCountPackageTag {
	long vmID;              // the virtual machine ID
	jobject accessibleContext;	// AccessibleContext containing AccessibleHypertext
	jint rLinkCount;        // link count return value
    } GetAccessibleHyperlinkCountPackage;
    
    // struct for sending a message to get the hypertext for an AccessibleContext
    // starting at a specified index in the document
    typedef struct GetAccessibleHypertextExtPackageTag {
	long vmID;              // the virtual machine ID
	jobject accessibleContext; // AccessibleContext with hypertext
	jint startIndex;        // start index in document 
	AccessibleHypertextInfo rAccessibleHypertextInfo; // returned hypertext
	BOOL rSuccess;          // whether call succeeded
    } GetAccessibleHypertextExtPackage;
    
    // struct for sending a message to get the nth hyperlink in a document;
    // maps to AccessibleHypertext.getLink
    typedef struct GetAccessibleHyperlinkPackageTag {
	long vmID;		// the virtual machine ID
	jobject hypertext;	// AccessibleHypertext
	jint linkIndex;		// hyperlink index
	AccessibleHyperlinkInfo rAccessibleHyperlinkInfo; // returned hyperlink
    } GetAccessibleHyperlinkPackage;
    
    // struct for sending a message to get the index into an array 
    // of hyperlinks that is associated with a character index in a
    // document; maps to AccessibleHypertext.getLinkIndex
    typedef struct GetAccessibleHypertextLinkIndexPackageTag {
	long vmID;              // the virtual machine ID
	jobject hypertext;	// AccessibleHypertext
	jint charIndex;         // character index in document
	jint rLinkIndex;        // returned hyperlink index
    } GetAccessibleHypertextLinkIndexPackage;


    /**
******************************************************
*  Accessible Key Bindings packages
******************************************************
*/

#define MAX_KEY_BINDINGS	8

    // keyboard character modifiers
#define ACCESSIBLE_SHIFT_KEYSTROKE			1
#define ACCESSIBLE_CONTROL_KEYSTROKE		2
#define ACCESSIBLE_META_KEYSTROKE			4
#define ACCESSIBLE_ALT_KEYSTROKE			8
#define ACCESSIBLE_ALT_GRAPH_KEYSTROKE		16
#define ACCESSIBLE_BUTTON1_KEYSTROKE		32
#define ACCESSIBLE_BUTTON2_KEYSTROKE		64
#define ACCESSIBLE_BUTTON3_KEYSTROKE		128

    // a key binding associates with a component
    typedef struct AccessibleKeyBindingInfoTag {
	jchar character;		// the key character
	jint modifiers;			// the key modifiers
    } AccessibleKeyBindingInfo;

    // all of the key bindings associated with a component
    typedef struct AccessibleKeyBindingsTag {
	int keyBindingsCount;	// number of key bindings
	AccessibleKeyBindingInfo keyBindingInfo[MAX_KEY_BINDINGS];	// the key bindings
    } AccessibleKeyBindings;

    // struct to get the key bindings associated with a component
    typedef struct GetAccessibleKeyBindingsPackageTag {
	long vmID;										// the virtual machine id
	jobject accessibleContext;						// the component
	AccessibleKeyBindings rAccessibleKeyBindings;	// the key bindings
    } GetAccessibleKeyBindingsPackage;

    /**
******************************************************
*  AccessibleIcon packages
******************************************************
*/
#define MAX_ICON_INFO 8

    // an icon assocated with a component
    typedef struct AccessibleIconInfoTag {
	wchar_t description[SHORT_STRING_SIZE];	// icon description
	jint height;							// icon height
	jint width;								// icon width
    } AccessibleIconInfo;

    // all of the icons associated with a component
    typedef struct AccessibleIconsTag {
	jint iconsCount;		// number of icons
	AccessibleIconInfo iconInfo[MAX_ICON_INFO];	// the icons
    } AccessibleIcons;

    // struct to get the icons associated with a component
    typedef struct GetAccessibleIconsPackageTag {
	long vmID;							// the virtual machine id
	jobject accessibleContext;			// the component
	AccessibleIcons rAccessibleIcons;	// the icons
    } GetAccessibleIconsPackage;


    /**
******************************************************
*  AccessibleAction packages
******************************************************
*/
#define MAX_ACTION_INFO 256
#define MAX_ACTIONS_TO_DO 32

    // an action assocated with a component
    typedef struct AccessibleActionInfoTag {
	wchar_t name[SHORT_STRING_SIZE];	// action name
    } AccessibleActionInfo;

    // all of the actions associated with a component
    typedef struct AccessibleActionsTag {
	jint actionsCount;		// number of actions
	AccessibleActionInfo actionInfo[MAX_ACTION_INFO];	// the action information
    } AccessibleActions;

    // struct for requesting the actions associated with a component
    typedef struct GetAccessibleActionsPackageTag {
	long vmID;
	jobject accessibleContext;					// the component
	AccessibleActions rAccessibleActions;		// the actions
    } GetAccessibleActionsPackage;

    // list of AccessibleActions to do
    typedef struct AccessibleActionsToDoTag {
	jint actionsCount;				// number of actions to do
	AccessibleActionInfo actions[MAX_ACTIONS_TO_DO];// the accessible actions to do
    } AccessibleActionsToDo;

    // struct for sending an message to do one or more actions
    typedef struct DoAccessibleActionsPackageTag {
	long vmID;						// the virtual machine ID
	jobject accessibleContext;		// component to do the action
	AccessibleActionsToDo actionsToDo;// the accessible actions to do
	BOOL rResult;                   // action return value
	jint failure;					// index of action that failed if rResult is FALSE
    } DoAccessibleActionsPackage;

    /**
******************************************************
*  AccessibleText packages
******************************************************
*/

    typedef struct GetAccessibleTextInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint x;
	jint y;
	AccessibleTextInfo rTextInfo;
    } GetAccessibleTextInfoPackage;

    typedef struct GetAccessibleTextItemsPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	AccessibleTextItemsInfo rTextItemsInfo;
    } GetAccessibleTextItemsPackage;

    typedef struct GetAccessibleTextSelectionInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	AccessibleTextSelectionInfo rTextSelectionItemsInfo;
    } GetAccessibleTextSelectionInfoPackage;

    typedef struct GetAccessibleTextAttributeInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	AccessibleTextAttributesInfo rAttributeInfo;
    } GetAccessibleTextAttributeInfoPackage;

    typedef struct GetAccessibleTextRectInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	AccessibleTextRectInfo rTextRectInfo;
    } GetAccessibleTextRectInfoPackage;

    typedef struct GetAccessibleTextLineBoundsPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	jint rLineStart;
	jint rLineEnd;
    } GetAccessibleTextLineBoundsPackage;

    typedef struct GetAccessibleTextRangePackageTag {
	long vmID;
	jobject AccessibleContext;
	jint start;
        jint end;
	wchar_t rText[MAX_STRING_SIZE];
    } GetAccessibleTextRangePackage;

    /**
******************************************************
*  
* Utility method packages
******************************************************
*/

    typedef struct SetTextContentsPackageTag {
	long vmID;
	jobject accessibleContext;		// the text field
	wchar_t text[MAX_STRING_SIZE];	// the text
	BOOL rResult;
    } SetTextContentsPackage; 

    typedef struct GetParentWithRolePackageTag {
	long vmID;
	jobject accessibleContext;
	wchar_t role[SHORT_STRING_SIZE];  // one of Accessible Roles above
	jobject rAccessibleContext;
    } GetParentWithRolePackage;

    typedef struct GetTopLevelObjectPackageTag {
	long vmID;
	jobject accessibleContext;
	jobject rAccessibleContext;
    } GetTopLevelObjectPackage;

    typedef struct GetParentWithRoleElseRootPackageTag {
	long vmID;
	jobject accessibleContext;
	wchar_t role[SHORT_STRING_SIZE];  // one of Accessible Roles above
	jobject rAccessibleContext;
    } GetParentWithRoleElseRootPackage;

    typedef struct GetObjectDepthPackageTag {
	long vmID;
	jobject accessibleContext;
	jint rResult;
    } GetObjectDepthPackage;

    typedef struct GetActiveDescendentPackageTag {
	long vmID;
	jobject accessibleContext;
	jobject rAccessibleContext;
    } GetActiveDescendentPackage;

    /**
******************************************************
*  AccessibleValue packages
******************************************************
*/

    typedef struct GetCurrentAccessibleValueFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	wchar_t rValue[SHORT_STRING_SIZE];
    } GetCurrentAccessibleValueFromContextPackage;

    typedef struct GetMaximumAccessibleValueFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	wchar_t rValue[SHORT_STRING_SIZE];
    } GetMaximumAccessibleValueFromContextPackage;

    typedef struct GetMinimumAccessibleValueFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	wchar_t rValue[SHORT_STRING_SIZE];
    } GetMinimumAccessibleValueFromContextPackage;


    /**
******************************************************
*  AccessibleSelection packages
******************************************************
*/

    typedef struct AddAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
    } AddAccessibleSelectionFromContextPackage;

    typedef struct ClearAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
    } ClearAccessibleSelectionFromContextPackage;

    typedef struct GetAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
        jint index;
	jobject rAccessibleContext;
    } GetAccessibleSelectionFromContextPackage;

    typedef struct GetAccessibleSelectionCountFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint rCount;
    } GetAccessibleSelectionCountFromContextPackage;

    typedef struct IsAccessibleChildSelectedFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
        jboolean rResult;
    } IsAccessibleChildSelectedFromContextPackage;

    typedef struct RemoveAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
    } RemoveAccessibleSelectionFromContextPackage;

    typedef struct SelectAllAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
    } SelectAllAccessibleSelectionFromContextPackage;


    /**
******************************************************
*  Java Event Notification Registration packages
******************************************************
*/

    typedef struct AddJavaEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
    } AddJavaEventNotificationPackage;

    typedef struct RemoveJavaEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
    } RemoveJavaEventNotificationPackage;


    /**
******************************************************
*  Accessibility Event Notification Registration packages
******************************************************
*/

    typedef struct AddAccessibilityEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
    } AddAccessibilityEventNotificationPackage;

    typedef struct RemoveAccessibilityEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
    } RemoveAccessibilityEventNotificationPackage;


    /**
******************************************************
*  Accessibility Property Change Event packages
******************************************************
*/

    typedef struct PropertyCaretChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	jint oldPosition;
	jint newPosition;
    } PropertyCaretChangePackage;

    typedef struct PropertyDescriptionChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldDescription[SHORT_STRING_SIZE];
	wchar_t newDescription[SHORT_STRING_SIZE];
    } PropertyDescriptionChangePackage;

    typedef struct PropertyNameChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldName[SHORT_STRING_SIZE];
	wchar_t newName[SHORT_STRING_SIZE];
    } PropertyNameChangePackage;

    typedef struct PropertySelectionChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } PropertySelectionChangePackage;

    typedef struct PropertyStateChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldState[SHORT_STRING_SIZE];
	wchar_t newState[SHORT_STRING_SIZE];
    } PropertyStateChangePackage;

    typedef struct PropertyTextChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } PropertyTextChangePackage;

    typedef struct PropertyValueChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldValue[SHORT_STRING_SIZE];
	wchar_t newValue[SHORT_STRING_SIZE];
    } PropertyValueChangePackage;

    typedef struct PropertyVisibleDataChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } PropertyVisibleDataChangePackage;

    typedef struct PropertyChildChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	jobject oldChildAccessibleContext;
	jobject newChildAccessibleContext;
    } PropertyChildChangePackage;

    typedef struct PropertyActiveDescendentChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	jobject oldActiveDescendentAccessibleContext;
	jobject newActiveDescendentAccessibleContext;
    } PropertyActiveDescendentChangePackage;


    // String format for newValue is:
    //	"type" one of "INSERT", "UPDATE" or "DELETE"
    //	"firstRow"
    //	"lastRow"
    //	"firstColumn"
    //	"lastColumn"
    //
    // oldValue is currently unused
    //
    typedef struct PropertyTableModelChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldValue[SHORT_STRING_SIZE];
	wchar_t newValue[SHORT_STRING_SIZE];
    } PropertyTableModelChangePackage;


    /**
******************************************************
*  Property Change Event packages
******************************************************
*/

    /*
      typedef struct PropertyChangePackageTag {
      long vmID;
      jobject Event;
      jobject AccessibleContextSource;
      char propertyName[SHORT_STRING_SIZE];
      char oldValue[SHORT_STRING_SIZE];	// PropertyChangeEvent().getOldValue().toString()
      char newValue[SHORT_STRING_SIZE];	// PropertyChangeEvent().getNewValue().toString()
      } PropertyChangePackage;
    */

    /**
******************************************************
*  Focus Event packages
******************************************************
*/

    typedef struct FocusGainedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } FocusGainedPackage; 

    typedef struct FocusLostPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } FocusLostPackage; 


    /**
******************************************************
*  Caret Event packages
******************************************************
*/

    typedef struct CaretUpdatePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } CaretUpdatePackage; 


    /**
******************************************************
*  Mouse Event packages
******************************************************
*/

    typedef struct MouseClickedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MouseClickedPackage; 

    typedef struct MouseEnteredPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MouseEnteredPackage;

    typedef struct MouseExitedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MouseExitedPackage;

    typedef struct MousePressedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MousePressedPackage;

    typedef struct MouseReleasedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MouseReleasedPackage;


    /**
******************************************************
*  Menu/PopupMenu Event packages
******************************************************
*/

    typedef struct MenuCanceledPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MenuCanceledPackage;

    typedef struct MenuDeselectedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MenuDeselectedPackage;

    typedef struct MenuSelectedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } MenuSelectedPackage;


    typedef struct PopupMenuCanceledPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } PopupMenuCanceledPackage;

    typedef struct PopupMenuWillBecomeInvisiblePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } PopupMenuWillBecomeInvisiblePackage;

    typedef struct PopupMenuWillBecomeVisiblePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
    } PopupMenuWillBecomeVisiblePackage;



    /**
******************************************************
*  Wrapping up all of the packages
******************************************************
*/

    /**
     *  What is the type of this package
     */
    typedef enum PackageType {

	cMemoryMappedFileCreatedPackage = 0x11000,

        // many of these will go away...
	cJavaVMCreatedPackage = 0x10000,
	cJavaVMDestroyedPackage,
	cWindowsATCreatedPackage,
	cWindowsATDestroyedPackage,
	cJavaVMPresentNotificationPackage,
	cWindowsATPresentNotificationPackage,
	
	cReleaseJavaObjectPackage = 1,
        cGetAccessBridgeVersionPackage = 2,

	cGetAccessibleContextFromHWNDPackage = 0x10,
        cIsJavaWindowPackage,
	cGetHWNDFromAccessibleContextPackage,

        cGetAccessibleContextAtPackage = 0x100,
        cGetAccessibleContextWithFocusPackage,
        cGetAccessibleContextInfoPackage,
	cGetAccessibleChildFromContextPackage,
	cGetAccessibleParentFromContextPackage,
	cIsSameObjectPackage,

	cGetAccessibleTextInfoPackage = 0x200,
	cGetAccessibleTextItemsPackage,
	cGetAccessibleTextSelectionInfoPackage,
	cGetAccessibleTextAttributeInfoPackage,
	cGetAccessibleTextRectInfoPackage,
	cGetAccessibleTextLineBoundsPackage,
	cGetAccessibleTextRangePackage,

	cGetCurrentAccessibleValueFromContextPackage = 0x300,
	cGetMaximumAccessibleValueFromContextPackage,
	cGetMinimumAccessibleValueFromContextPackage,

	cAddAccessibleSelectionFromContextPackage = 0x400,
	cClearAccessibleSelectionFromContextPackage,
	cGetAccessibleSelectionFromContextPackage,
	cGetAccessibleSelectionCountFromContextPackage,
	cIsAccessibleChildSelectedFromContextPackage,
	cRemoveAccessibleSelectionFromContextPackage,
	cSelectAllAccessibleSelectionFromContextPackage,

	cAddJavaEventNotificationPackage = 0x900,
	cRemoveJavaEventNotificationPackage,
	cAddAccessibilityEventNotificationPackage,
	cRemoveAccessibilityEventNotificationPackage,

	cPropertyChangePackage = 0x1000,

	cFocusGainedPackage = 0x1010,
	cFocusLostPackage,

	cCaretUpdatePackage = 0x1020,

	cMouseClickedPackage = 0x1030,
	cMouseEnteredPackage,
	cMouseExitedPackage,
	cMousePressedPackage,
	cMouseReleasedPackage,

	cMenuCanceledPackage = 0x1040,
	cMenuDeselectedPackage,
	cMenuSelectedPackage,
	cPopupMenuCanceledPackage,
	cPopupMenuWillBecomeInvisiblePackage,
	cPopupMenuWillBecomeVisiblePackage,

   	cPropertyCaretChangePackage = 0x1100,
   	cPropertyDescriptionChangePackage,
   	cPropertyNameChangePackage,
   	cPropertySelectionChangePackage,
   	cPropertyStateChangePackage,
   	cPropertyTextChangePackage,
   	cPropertyValueChangePackage,
   	cPropertyVisibleDataChangePackage,
   	cPropertyChildChangePackage,
   	cPropertyActiveDescendentChangePackage,


	// AccessibleTable
	cGetAccessibleTableInfoPackage = 0x1200,
	cGetAccessibleTableCellInfoPackage,

	cGetAccessibleTableRowHeaderPackage,
	cGetAccessibleTableColumnHeaderPackage,

	cGetAccessibleTableRowDescriptionPackage,
        cGetAccessibleTableColumnDescriptionPackage,

	cGetAccessibleTableRowSelectionCountPackage,
        cIsAccessibleTableRowSelectedPackage,
        cGetAccessibleTableRowSelectionsPackage,

	cGetAccessibleTableColumnSelectionCountPackage,
        cIsAccessibleTableColumnSelectedPackage,
        cGetAccessibleTableColumnSelectionsPackage,

	cGetAccessibleTableRowPackage,
	cGetAccessibleTableColumnPackage,
	cGetAccessibleTableIndexPackage,

	cPropertyTableModelChangePackage,


	// AccessibleRelationSet
	cGetAccessibleRelationSetPackage = 0x1300,

	// AccessibleHypertext
	cGetAccessibleHypertextPackage = 0x1400,
	cActivateAccessibleHyperlinkPackage,
        cGetAccessibleHyperlinkCountPackage,
	cGetAccessibleHypertextExtPackage,
	cGetAccessibleHypertextLinkIndexPackage,
	cGetAccessibleHyperlinkPackage,

	// Accessible KeyBinding, Icon and Action
	cGetAccessibleKeyBindingsPackage = 0x1500,
	cGetAccessibleIconsPackage,
	cGetAccessibleActionsPackage,
	cDoAccessibleActionsPackage,

	// Utility methods
	cSetTextContentsPackage = 0x1600,
	cGetParentWithRolePackage,
	cGetTopLevelObjectPackage,
	cGetParentWithRoleElseRootPackage,
	cGetObjectDepthPackage,
	cGetActiveDescendentPackage


    } PackageType;


    /**
     *  Union of all package contents
     */
    typedef union AllPackagesTag {

	// Initial Rendezvous packages
	MemoryMappedFileCreatedPackage memoryMappedFileCreatedPackage;

	JavaVMCreatedPackage javaVMCreatedPackage;
	JavaVMDestroyedPackage javaVMDestroyedPackage;
	WindowsATCreatedPackage windowsATCreatedPackage;
	WindowsATDestroyedPackage windowsATDestroyedPackage;
	JavaVMPresentNotificationPackage javaVMPresentNotificationPackage;
	WindowsATPresentNotificationPackage windowsATPresentNotificationPackage;

	// Core packages
	ReleaseJavaObjectPackage releaseJavaObject;
        GetAccessBridgeVersionPackage getAccessBridgeVersion;

	// Window packages
	GetAccessibleContextFromHWNDPackage getAccessibleContextFromHWND;
	GetHWNDFromAccessibleContextPackage getHWNDFromAccessibleContext;

	// AccessibleContext packages
        GetAccessibleContextAtPackage getAccessibleContextAt;
        GetAccessibleContextWithFocusPackage getAccessibleContextWithFocus;
        GetAccessibleContextInfoPackage getAccessibleContextInfo;
	GetAccessibleChildFromContextPackage getAccessibleChildFromContext;
	GetAccessibleParentFromContextPackage getAccessibleParentFromContext;

	// AccessibleText packages
	GetAccessibleTextInfoPackage getAccessibleTextInfo;
	GetAccessibleTextItemsPackage getAccessibleTextItems;
	GetAccessibleTextSelectionInfoPackage getAccessibleTextSelectionInfo;
	GetAccessibleTextAttributeInfoPackage getAccessibleTextAttributeInfo;
	GetAccessibleTextRectInfoPackage getAccessibleTextRectInfo;
        GetAccessibleTextLineBoundsPackage getAccessibleTextLineBounds;
        GetAccessibleTextRangePackage getAccessibleTextRange;

	// AccessibleValue packages
        GetCurrentAccessibleValueFromContextPackage getCurrentAccessibleValueFromContext;
        GetMaximumAccessibleValueFromContextPackage getMaximumAccessibleValueFromContext;
        GetMinimumAccessibleValueFromContextPackage getMinimumAccessibleValueFromContext;

	// AccessibleSelection packages
        AddAccessibleSelectionFromContextPackage addAccessibleSelectionFromContext;
        ClearAccessibleSelectionFromContextPackage clearAccessibleSelectionFromContext;
        GetAccessibleSelectionFromContextPackage getAccessibleSelectionFromContext;
        GetAccessibleSelectionCountFromContextPackage getAccessibleSelectionCountFromContext;
        IsAccessibleChildSelectedFromContextPackage isAccessibleChildSelectedFromContext;
        RemoveAccessibleSelectionFromContextPackage removeAccessibleSelectionFromContext;
        SelectAllAccessibleSelectionFromContextPackage selectAllAccessibleSelectionFromContext;

	// Event Notification Registration packages
	AddJavaEventNotificationPackage addJavaEventNotification;
	RemoveJavaEventNotificationPackage removeJavaEventNotification;
	AddAccessibilityEventNotificationPackage addAccessibilityEventNotification;
	RemoveAccessibilityEventNotificationPackage removeAccessibilityEventNotification;

	// Event contents packages
        //	PropertyChangePackage propertyChange;
        PropertyCaretChangePackage propertyCaretChangePackage;
        PropertyDescriptionChangePackage propertyDescriptionChangePackage;
        PropertyNameChangePackage propertyNameChangePackage;
        PropertySelectionChangePackage propertySelectionChangePackage;
        PropertyStateChangePackage propertyStateChangePackage;
        PropertyTextChangePackage propertyTextChangePackage;
        PropertyValueChangePackage propertyValueChangePackage;
        PropertyVisibleDataChangePackage propertyVisibleDataChangePackage;
        PropertyChildChangePackage propertyChildChangePackage;
        PropertyActiveDescendentChangePackage propertyActiveDescendentChangePackage;

	PropertyTableModelChangePackage propertyTableModelChangePackage;

	FocusGainedPackage focusGained;
	FocusLostPackage focusLost;

	CaretUpdatePackage caretUpdate;

	MouseClickedPackage mouseClicked;
	MouseEnteredPackage mouseEntered;
	MouseExitedPackage mouseExited;
	MousePressedPackage mousePressed;
	MouseReleasedPackage mouseReleased;

	MenuCanceledPackage menuCanceled;
	MenuDeselectedPackage menuDeselected;
	MenuSelectedPackage menuSelected;
	PopupMenuCanceledPackage popupMenuCanceled;
	PopupMenuWillBecomeInvisiblePackage popupMenuWillBecomeInvisible;
	PopupMenuWillBecomeVisiblePackage popupMenuWillBecomeVisible;

	// utility methods
	SetTextContentsPackage _setTextContents;
	GetParentWithRolePackage _getParentWithRole;
	GetTopLevelObjectPackage _getTopLevelObject;
	GetParentWithRoleElseRootPackage _getParentWithRoleElseRoot;
	GetObjectDepthPackage _getObjectDepth;
	GetActiveDescendentPackage _getActiveDescendent;

    } AllPackages;


    /**
     *  Union of all Java-initiated package contents
     */
    typedef union JavaInitiatedPackagesTag {

	// Initial Rendezvous packages
	JavaVMCreatedPackage javaVMCreatedPackage;
	JavaVMDestroyedPackage javaVMDestroyedPackage;
	JavaVMPresentNotificationPackage javaVMPresentNotificationPackage;

	// Event contents packages
        //	PropertyChangePackage propertyChange;
        PropertyCaretChangePackage propertyCaretChangePackage;
        PropertyDescriptionChangePackage propertyDescriptionChangePackage;
        PropertyNameChangePackage propertyNameChangePackage;
        PropertySelectionChangePackage propertySelectionChangePackage;
        PropertyStateChangePackage propertyStateChangePackage;
        PropertyTextChangePackage propertyTextChangePackage;
        PropertyValueChangePackage propertyValueChangePackage;
        PropertyVisibleDataChangePackage propertyVisibleDataChangePackage;
        PropertyChildChangePackage propertyChildChangePackage;
        PropertyActiveDescendentChangePackage propertyActiveDescendentChangePackage;

	PropertyTableModelChangePackage propertyTableModelChangePackage;

	FocusGainedPackage focusGained;
	FocusLostPackage focusLost;

	CaretUpdatePackage caretUpdate;

	MouseClickedPackage mouseClicked;
	MouseEnteredPackage mouseEntered;
	MouseExitedPackage mouseExited;
	MousePressedPackage mousePressed;
	MouseReleasedPackage mouseReleased;

	MenuCanceledPackage menuCanceled;
	MenuDeselectedPackage menuDeselected;
	MenuSelectedPackage menuSelected;
	PopupMenuCanceledPackage popupMenuCanceled;
	PopupMenuWillBecomeInvisiblePackage popupMenuWillBecomeInvisible;
	PopupMenuWillBecomeVisiblePackage popupMenuWillBecomeVisible;

    } JavaInitiatedPackages;


    /**
     *  Union of all Windows-initiated package contents
     */
    typedef union WindowsInitiatedPackagesTag {

	// Initial Rendezvous packages
	MemoryMappedFileCreatedPackage memoryMappedFileCreatedPackage;

	WindowsATCreatedPackage windowsATCreatedPackage;
	WindowsATDestroyedPackage windowsATDestroyedPackage;
	WindowsATPresentNotificationPackage windowsATPresentNotificationPackage;

	// Core packages
	ReleaseJavaObjectPackage releaseJavaObject;
        GetAccessBridgeVersionPackage getAccessBridgeVersion;

	// Window packages
	GetAccessibleContextFromHWNDPackage getAccessibleContextFromHWND;
	GetHWNDFromAccessibleContextPackage getHWNDFromAccessibleContext;

	// AccessibleContext packages
        GetAccessibleContextAtPackage getAccessibleContextAt;
        GetAccessibleContextWithFocusPackage getAccessibleContextWithFocus;
        GetAccessibleContextInfoPackage getAccessibleContextInfo;
	GetAccessibleChildFromContextPackage getAccessibleChildFromContext;
	GetAccessibleParentFromContextPackage getAccessibleParentFromContext;

	// AccessibleText packages
	GetAccessibleTextInfoPackage getAccessibleTextInfo;
	GetAccessibleTextItemsPackage getAccessibleTextItems;
	GetAccessibleTextSelectionInfoPackage getAccessibleTextSelectionInfo;
	GetAccessibleTextAttributeInfoPackage getAccessibleTextAttributeInfo;
	GetAccessibleTextRectInfoPackage getAccessibleTextRectInfo;
        GetAccessibleTextLineBoundsPackage getAccessibleTextLineBounds;
        GetAccessibleTextRangePackage getAccessibleTextRange;

	// AccessibleValue packages
        GetCurrentAccessibleValueFromContextPackage getCurrentAccessibleValueFromContext;
        GetMaximumAccessibleValueFromContextPackage getMaximumAccessibleValueFromContext;
        GetMinimumAccessibleValueFromContextPackage getMinimumAccessibleValueFromContext;

	// AccessibleSelection packages
        AddAccessibleSelectionFromContextPackage addAccessibleSelectionFromContext;
        ClearAccessibleSelectionFromContextPackage clearAccessibleSelectionFromContext;
        GetAccessibleSelectionFromContextPackage getAccessibleSelectionFromContext;
        GetAccessibleSelectionCountFromContextPackage getAccessibleSelectionCountFromContext;
        IsAccessibleChildSelectedFromContextPackage isAccessibleChildSelectedFromContext;
        RemoveAccessibleSelectionFromContextPackage removeAccessibleSelectionFromContext;
        SelectAllAccessibleSelectionFromContextPackage selectAllAccessibleSelectionFromContext;

        // Event Notification Registration packages
	AddJavaEventNotificationPackage addJavaEventNotification;
	RemoveJavaEventNotificationPackage removeJavaEventNotification;
	AddAccessibilityEventNotificationPackage addAccessibilityEventNotification;
	RemoveAccessibilityEventNotificationPackage removeAccessibilityEventNotification;

	// AccessibleTable
	GetAccessibleTableInfoPackage _getAccessibleTableInfo;
	GetAccessibleTableCellInfoPackage _getAccessibleTableCellInfo;

	GetAccessibleTableRowHeaderPackage _getAccessibleTableRowHeader;
	GetAccessibleTableColumnHeaderPackage _getAccessibleTableColumnHeader;

	GetAccessibleTableRowDescriptionPackage _getAccessibleTableRowDescription;
	GetAccessibleTableColumnDescriptionPackage _getAccessibleTableColumnDescription;

	GetAccessibleTableRowSelectionCountPackage _getAccessibleTableRowSelectionCount;
        IsAccessibleTableRowSelectedPackage _isAccessibleTableRowSelected;
        GetAccessibleTableRowSelectionsPackage _getAccessibleTableRowSelections;

        GetAccessibleTableColumnSelectionCountPackage _getAccessibleTableColumnSelectionCount;
        IsAccessibleTableColumnSelectedPackage _isAccessibleTableColumnSelected;
        GetAccessibleTableColumnSelectionsPackage _getAccessibleTableColumnSelections;

	GetAccessibleTableRowPackage _getAccessibleTableRow;
	GetAccessibleTableColumnPackage _getAccessibleTableColumn;
	GetAccessibleTableIndexPackage _getAccessibleTableIndex;

	// AccessibleRelationSet
	GetAccessibleRelationSetPackage _getAccessibleRelationSet;

	// AccessibleHypertext
	GetAccessibleHypertextPackage _getAccessibleHypertext;
	ActivateAccessibleHyperlinkPackage _activateAccessibleHyperlink;
        GetAccessibleHyperlinkCountPackage _getAccessibleHyperlinkCount;
	GetAccessibleHypertextExtPackage _getAccessibleHypertextExt;
	GetAccessibleHypertextLinkIndexPackage _getAccessibleHypertextLinkIndex;
	GetAccessibleHyperlinkPackage _getAccessibleHyperlink;

	// Accessible KeyBindings, Icons and Actions
	GetAccessibleKeyBindingsPackage _getAccessibleKeyBindings;
	GetAccessibleIconsPackage _getAccessibleIcons;
	GetAccessibleActionsPackage _getAccessibleActions;
	DoAccessibleActionsPackage _doAccessibleActions;

	IsSameObjectPackage _isSameObject;

	// utility methods
	SetTextContentsPackage _setTextContents;
	GetParentWithRolePackage _getParentWithRole;
	GetTopLevelObjectPackage _getTopLevelObject;
	GetParentWithRoleElseRootPackage _getParentWithRoleElseRoot;
	GetObjectDepthPackage _getObjectDepth;
	GetActiveDescendentPackage _getActiveDescendent;


    } WindowsInitiatedPackages;


#ifdef __cplusplus
}
#endif

#endif
